# This is here to import it once, which improves the speed of launch when in debug-mode
try:
    import transformer_engine  # noqa
except ImportError:
    pass

from nemo.collections.llm import peft, tokenizer
from nemo.collections.llm.api import export_ckpt, finetune, import_ckpt, pretrain, train, validate
from nemo.collections.llm.gpt.data import (
    DollyDataModule,
    FineTuningDataModule,
    MockDataModule,
    PreTrainingDataModule,
    SquadDataModule,
)
from nemo.collections.llm.gpt.data.api import dolly, mock, squad
from nemo.collections.llm.gpt.model import (
    CodeGemmaConfig2B,
    CodeGemmaConfig7B,
    CodeLlamaConfig7B,
    CodeLlamaConfig13B,
    CodeLlamaConfig34B,
    CodeLlamaConfig70B,
    GemmaConfig,
    GemmaConfig2B,
    GemmaConfig7B,
    GemmaModel,
    GPTConfig,
    GPTModel,
    Llama2Config7B,
    Llama2Config13B,
    Llama2Config70B,
    Llama3Config8B,
    Llama3Config70B,
    LlamaConfig,
    LlamaModel,
    MaskedTokenLossReduction,
    MistralConfig7B,
    MistralModel,
    MixtralConfig8x7B,
    MixtralModel,
    gpt_data_step,
    gpt_forward_step,
)
from nemo.collections.llm.recipes import *  # noqa
from nemo.utils import logging

try:
    from nemo.collections.llm.api import deploy
except ImportError as error:
    deploy = None
    logging.warning(f"The deploy module could not be imported: {error}")

__all__ = [
    "MockDataModule",
    "GPTModel",
    "GPTConfig",
    "gpt_data_step",
    "gpt_forward_step",
    "MaskedTokenLossReduction",
    "MistralConfig7B",
    "MistralModel",
    "MixtralConfig8x7B",
    "MixtralModel",
    "LlamaConfig",
    "Llama2Config7B",
    "Llama2Config13B",
    "Llama2Config70B",
    "Llama3Config8B",
    "Llama3Config70B",
    "CodeLlamaConfig7B",
    "CodeLlamaConfig13B",
    "CodeLlamaConfig34B",
    "CodeLlamaConfig70B",
    "LlamaModel",
    "GemmaConfig",
    "GemmaConfig2B",
    "GemmaConfig7B",
    "CodeGemmaConfig2B",
    "CodeGemmaConfig7B",
    "GemmaModel",
    "PreTrainingDataModule",
    "FineTuningDataModule",
    "SquadDataModule",
    "DollyDataModule",
    "train",
    "import_ckpt",
    "export_ckpt",
    "pretrain",
    "validate",
    "finetune",
    "tokenizer",
    "mock",
    "squad",
    "dolly",
    "peft",
]

# add 'deploy' to __all__ if it was successfully imported
if deploy is not None:
    __all__.append("deploy")
